using System;
using System.Collections.Generic;
using System.Globalization;
using System.Text;

namespace Vevo.Shared.Utilities
{
    public static class ConvertUtilities
    {
        #region Private

        private static bool IsEmptyObject( object source )
        {
            if (source == null ||
                String.IsNullOrEmpty( source.ToString() ) ||
                (source is DBNull))
            {
                return true;
            }
            else
            {
                return false;
            }
        }

        private static CultureInfo USCultureInfo()
        {
            return new CultureInfo( "en-US" );
        }

        private static string ReplaceEngNumberStirng( string number )
        {
            return number.Replace( ',', '.' );
        }

        #endregion


        #region Public Methods

        public static string ToString( object source )
        {
            if (IsEmptyObject( source ))
                return String.Empty;
            else
                return source.ToString();
        }

        public static bool ToBoolean( object source )
        {
            if (IsEmptyObject( source ))
                return false;
            else
                return Convert.ToBoolean( source );
        }


        public static string ToYesNo( object source )
        {
            if (IsEmptyObject( source ))
                return "No";
            else
                return Convert.ToBoolean( source ) ? "Yes" : "No";
        }

        public static string ToYN( object source )
        {
            if (IsEmptyObject( source ))
                return "N";
            else
                return Convert.ToBoolean( source ) ? "Y" : "N";
        }

        public static decimal ToDecimal( object source )
        {
            if (IsEmptyObject( source ))
                return 0.0m;
            else
                return Convert.ToDecimal( source );
        }

        public static double ToDouble( object source )
        {
            if (IsEmptyObject( source ))
                return 0.0;
            else
                return Convert.ToDouble( source );
        }

        public static int ToInt32( object source )
        {
            if (IsEmptyObject( source ))
                return 0;
            else
                return Convert.ToInt32( source );
        }

        public static DateTime ToDateTime( object source )
        {
            if (IsEmptyObject( source ))
                return new DateTime();
            else
                return Convert.ToDateTime( source );
        }

        public static double EnglishStringToDouble( string number )
        {
            return Convert.ToDouble( ReplaceEngNumberStirng( number ), USCultureInfo() );
        }

        public static decimal EnglishStringToDecimal( string number )
        {
            return Convert.ToDecimal( ReplaceEngNumberStirng( number ), USCultureInfo() );
        }

        #endregion

    }
}
