using System;
using System.Collections.Generic;
using System.Text;

using Vevo.Domain;
using Vevo.Domain.Base;
using Vevo.Shared.DataAccess;
using Vevo.Base.Domain;

namespace Vevo.Base.Data.Implementation
{
    public abstract class BaseListedDataAccess<IDType, DataType>
    {
        #region Private

        private void CreateItems( IDType parentID, IEntityList<DataType> list )
        {
            foreach (DataType item in list.GetAddedList())
                Create( parentID, item );
        }

        private void UpdateItems( IDType parentID, IEntityList<DataType> list )
        {
            foreach (DataType item in list.GetUpdatedList())
                Update( parentID, item );
        }

        private void DeleteItems( IDType parentID, IEntityList<DataType> list )
        {
            foreach (DataType item in list.GetRemovedList())
                Delete( parentID, item );
        }

        private void DeleteAllItems( IDType parentID )
        {
            DeleteAllItems( parentID );
        }

        #endregion


        #region Protected

        protected abstract void Create( IDType parentID, DataType item );
        protected abstract void Update( IDType parentID, DataType item );
        protected abstract void Delete( IDType parentID, DataType item );
        //protected abstract void DeleteAll( IDType parentID );

        #endregion


        #region Public Methods

        public virtual void SaveAll(
            IDType parentID,
            IEntityList<DataType> list )
        {
            using (DataTransactionScope transaction = new DataTransactionScope())
            {
                DeleteItems( parentID, list );
                CreateItems( parentID, list );
                UpdateItems( parentID, list );

                list.AcceptChanges();

                transaction.Complete();
            }
        }

        #endregion

    }
}
