using System;
using System.Collections.Generic;
using System.Text;

namespace Vevo.Base.Domain
{
    public class Address
    {
        #region Factory Class

        public class Factory
        {
            private string _firstName = String.Empty;
            private string _lastName = String.Empty;
            private string _company = String.Empty;
            private string _address1 = String.Empty;
            private string _address2 = String.Empty;
            private string _city = String.Empty;
            private string _state = String.Empty;
            private string _zip = String.Empty;
            private string _country = String.Empty;
            private string _phone = String.Empty;
            private string _fax = String.Empty;

            public Factory WithFirstName( string value ) { _firstName = value; return this; }
            public Factory WithLastName( string value ) { _lastName = value; return this; }
            public Factory WithCompany( string value ) { _company = value; return this; }
            public Factory WithAddress1( string value ) { _address1 = value; return this; }
            public Factory WithAddress2( string value ) { _address2 = value; return this; }
            public Factory WithCity( string value ) { _city = value; return this; }
            public Factory WithState( string value ) { _state = value; return this; }
            public Factory WithZip( string value ) { _zip = value; return this; }
            public Factory WithCountry( string value ) { _country = value; return this; }
            public Factory WithPhone( string value ) { _phone = value; return this; }
            public Factory WithFax( string value ) { _fax = value; return this; }
            public Address Create()
            {
                return new Address( _firstName, _lastName, _company, _address1, _address2,
                    _city, _state, _zip, _country, _phone, _fax );
            }
            public ShippingAddress CreateShippingAddress( bool residential )
            {
                return new ShippingAddress( Create(), residential );
            }
        }

        public static Factory GetFactory() { return new Factory(); }

        #endregion


        #region Private

        private string _firstName = String.Empty;
        private string _lastName = String.Empty;
        private string _company = String.Empty;
        private string _address1 = String.Empty;
        private string _address2 = String.Empty;
        private string _city = String.Empty;
        private string _state = String.Empty;
        private string _zip = String.Empty;
        private string _country = String.Empty;
        private string _phone = String.Empty;
        private string _fax = String.Empty;

        #endregion


        #region Constructors

        public Address()
        {
        }

        public Address( Address address )
        {
            _firstName = address._firstName;
            _lastName = address._lastName;
            _company = address._company;
            _address1 = address._address1;
            _address2 = address._address2;
            _city = address._city;
            _state = address._state;
            _zip = address._zip;
            _country = address._country;
            _phone = address._phone;
            _fax = address._fax;
        }

        public Address(
            string firstName,
            string lastName,
            string company,
            string address1,
            string address2,
            string city,
            string state,
            string zip,
            string country,
            string phone,
            string fax )
        {
            _firstName = firstName;
            _lastName = lastName;
            _company = company;
            _address1 = address1;
            _address2 = address2;
            _city = city;
            _state = state;
            _zip = zip;
            _country = country;
            _phone = phone;
            _fax = fax;

            if (_firstName == null) _firstName = String.Empty;
            if (_lastName == null) _lastName = String.Empty;
            if (_company == null) _company = String.Empty;
            if (_address1 == null) _address1 = String.Empty;
            if (_address2 == null) _address2 = String.Empty;
            if (_city == null) _city = String.Empty;
            if (_state == null) _state = String.Empty;
            if (_zip == null) _zip = String.Empty;
            if (_country == null) _country = String.Empty;
            if (_phone == null) _phone = String.Empty;
            if (_fax == null) _fax = String.Empty;
        }

        #endregion


        #region Public Properties

        public string FirstName { get { return _firstName; } }
        public string LastName { get { return _lastName; } }
        public string Company { get { return _company; } }
        public string Address1 { get { return _address1; } }
        public string Address2 { get { return _address2; } }
        public string City { get { return _city; } }
        public string State { get { return _state; } }
        public string Zip { get { return _zip; } }
        public string Country { get { return _country; } }
        public string Phone { get { return _phone; } }
        public string Fax { get { return _fax; } }

        #endregion


        #region Public Methods

        public override bool Equals( object obj )
        {
            Address rhs = (Address) obj;

            return FirstName == rhs.FirstName
                && LastName == rhs.LastName
                && Company == rhs.Company
                && Address1 == rhs.Address1
                && Address2 == rhs.Address2
                && City == rhs.City
                && State == rhs.State
                && Zip == rhs.Zip
                && Country == rhs.Country
                && Phone == rhs.Phone
                && Fax == rhs.Fax;
        }

        public override int GetHashCode()
        {
            return _firstName.GetHashCode()
                ^ _lastName.GetHashCode()
                ^ _company.GetHashCode()
                ^ _address1.GetHashCode()
                ^ _address2.GetHashCode()
                ^ _city.GetHashCode()
                ^ _state.GetHashCode()
                ^ _zip.GetHashCode()
                ^ _country.GetHashCode()
                ^ _phone.GetHashCode()
                ^ _fax.GetHashCode();
        }

        #endregion

    }
}
