﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Vevo.Base.Data.Caching;
using Vevo.Domain;
using Vevo.Domain.DataInterfaces;
using Vevo.Domain.Blogs;
using System.Web.UI;
using Vevo.Shared.Utilities;
using Vevo.Base.Domain;

namespace Vevo.Data
{
    public class BlogCommentRepository : IBlogCommentRepository
    {
        #region Private

        private const string RepositoryName = "BlogCommentRepository";

        private const string GetOneSignature = "GetOne";
        private const string GetAllByBlogIDSignature = "GetAllByBlogID";
        private const string SearchBlogCommentSignature = "SearchBlogComment";
        private const string GetTableSchemaSignature = "GetTableSchema";

        private DataCache _dataCache = new DataCache( RepositoryName );
        private ICacheDependencyUtility _cacheDependency = new CacheDependencyUtility( new string[] { "BlogComment" } );
        private BlogCommentRepositoryNoCache _repository = new BlogCommentRepositoryNoCache();

        #endregion

        #region Constructors

        public BlogCommentRepository()
        {
        }

        public BlogCommentRepository( ICacheDependencyUtility cacheDependency )
        {
            _cacheDependency = cacheDependency;
        }

        #endregion

        #region Public Methods

        public BlogComment Save( BlogComment blogComment )
        {
            BlogComment result = _repository.Save( blogComment );
            _dataCache.ClearRepositoryCache();

            return result;
        }

        public void Delete( string blogCommentID )
        {
            _repository.Delete( blogCommentID );
            _dataCache.ClearRepositoryCache();
        }

        public BlogComment GetOne( string blogCommentID )
        {
            BlogComment result = (BlogComment) _dataCache.GetValue( GetOneSignature, _cacheDependency, blogCommentID );

            if (result == null)
            {
                result = _repository.GetOne( blogCommentID );
                _dataCache.SetValue( GetOneSignature, result, blogCommentID );
            }

            return result;
        }

        public IList<BlogComment> GetAllByBlogID( string blogCommentID, string sortBy )
        {
            IList<BlogComment> list = (IList<BlogComment>) _dataCache.GetValue(
                GetAllByBlogIDSignature, _cacheDependency, blogCommentID, sortBy );

            if (list == null)
            {
                list = _repository.GetAllByBlogID( blogCommentID, sortBy );
                _dataCache.SetValue( GetAllByBlogIDSignature, list, blogCommentID, sortBy );
            }

            return list;
        }

        public IList<TableSchemaItem> GetTableSchema()
        {
            IList<TableSchemaItem> list = (IList<TableSchemaItem>) _dataCache.GetValue( GetTableSchemaSignature, _cacheDependency );

            if (list == null)
            {
                list = _repository.GetTableSchema();
                _dataCache.SetValue(
                    GetTableSchemaSignature,
                    list
                    );
            }

            return list;
        }

        public IList<BlogComment> SearchBlogComment(
            string blogID,
            string sortBy,
            SearchFilter searchFilter,
            int startIndex,
            int endIndex,
            out int howManyItems,
            BoolFilter isEnabled )
        {
            howManyItems = 0;
            Pair result = (Pair) _dataCache.GetValue(
                SearchBlogCommentSignature,
                _cacheDependency,
                blogID,
                sortBy,
                searchFilter.ToString(),
                startIndex,
                endIndex,
                isEnabled.ToString()
                );

            if (result == null)
            {
                result = new Pair();
                result.First = _repository.SearchBlogComment(blogID,
                    sortBy, searchFilter, startIndex, endIndex, out howManyItems, isEnabled );
                result.Second = howManyItems;

                _dataCache.SetValue(
                    SearchBlogCommentSignature,
                    result,
                    blogID,
                    sortBy,
                    searchFilter.ToString(),
                    startIndex,
                    isEnabled.ToString()
                    );
            }

            howManyItems = ConvertUtilities.ToInt32( result.Second );
            return (IList<BlogComment>) result.First;
        }

        public IList<BlogComment> SearchBlogComment(
            string blogID,
            string sortBy,
            int startIndex,
            int endIndex,
            out int howManyItems,
            BoolFilter isEnabled )
        {
            howManyItems = 0;
            Pair result = (Pair) _dataCache.GetValue(
                SearchBlogCommentSignature,
                _cacheDependency,
                blogID,
                sortBy,
                startIndex,
                endIndex,
                isEnabled.ToString()
                );

            if (result == null)
            {
                result = new Pair();
                result.First = _repository.SearchBlogComment( blogID,
                    sortBy, startIndex, endIndex, out howManyItems, isEnabled );
                result.Second = howManyItems;

                _dataCache.SetValue(
                    SearchBlogCommentSignature,
                    result,
                    blogID,
                    sortBy,
                    startIndex,
                    isEnabled.ToString()
                    );
            }

            howManyItems = ConvertUtilities.ToInt32( result.Second );
            return (IList<BlogComment>) result.First;
        }

        #endregion
    }
}
