﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data;
using Vevo.Shared.DataAccess;
using Vevo.Shared.Utilities;
using Vevo.Domain.GoogleFeed;
using Vevo.Domain.DataInterfaces;
using System.Collections;
using Vevo.Domain;
using Vevo.Base.Data;
using Vevo.Base.Domain;

namespace Vevo.Data.Implementation
{
    internal class GoogleSpecMappingAccess
    {
        #region Private

        private GoogleSpecMapping CreateOneFromRow( DataRow row )
        {
            GoogleSpecMapping specMapping = new GoogleSpecMapping();
            specMapping.GoogleFeedTagID = row["GoogleFeedTagID"].ToString();
            specMapping.TagName = row["TagName"].ToString();
            specMapping.GoogleFeedTagValueID = row["GoogleFeedTagValueID"].ToString();
            specMapping.ValueName = row["ValueName"].ToString();
            specMapping.SpecificationItemID = row["SpecificationItemID"].ToString();
            specMapping.DisplayName = row["DisplayName"].ToString();
            specMapping.SpecificationGroupName = row["SpecificationGroupName"].ToString();
            specMapping.Value = row["Value"].ToString();

            if (row.Table.Columns.Contains( "SpecificationGroupDisplayName" ))
                specMapping.SpecificationGroupDisplayName = row["SpecificationGroupDisplayName"].ToString();

            return specMapping;
        }

        private IList<GoogleSpecMapping> ConvertTableToList( DataTable table )
        {
            IList<GoogleSpecMapping> list = new List<GoogleSpecMapping>();

            foreach (DataRow row in table.Rows)
                list.Add( CreateOneFromRow( row ) );

            return list;
        }

        private IList<GoogleSpecMapping> ConvertTableToList( DataTable table, int startIndex, int endIndex )
        {
            IList<GoogleSpecMapping> list = new List<GoogleSpecMapping>();

            for (int i = startIndex; i <= endIndex && i < table.Rows.Count; i++)
                list.Add( CreateOneFromRow( table.Rows[i] ) );

            return list;
        }

        private GoogleSpecMapping Create( Culture culture, GoogleSpecMapping googleSpecMapping )
        {
            int rowAffect = DataAccess.ExecuteNonQuery(
                "INSERT INTO GoogleSpecMapping (GoogleFeedTagValueID ,SpecificationItemID, Value ) " +
                "VALUES ( @GoogleFeedTagValueID, @SpecificationItemID, @Value );",
                DataAccess.CreateParameterString( googleSpecMapping.GoogleFeedTagValueID ),
                DataAccess.CreateParameterString( googleSpecMapping.SpecificationItemID ),
                DataAccess.CreateParameterString( googleSpecMapping.Value ) );

            if (rowAffect == 1)
                return GetOne( culture, googleSpecMapping.GoogleFeedTagValueID, googleSpecMapping.Value );
            else
                return GoogleSpecMapping.Null;
        }

        private int GetGoogleSpecMappingCount( Culture culture )
        {
            string count = DataAccess.ExecuteScalar(
                "SELECT COUNT (Value) AS [RowCount] " +
                "FROM GoogleSpecMapping " +
                "LEFT JOIN " +
                "    (SELECT GoogleFeedTagValueID, ValueName FROM GoogleFeedTagValue) AS GoogleFeedTagValue " +
                "     ON GoogleSpecMapping.GoogleFeedTagValueID = GoogleFeedTagValue.GoogleFeedTagValueID " +
                "     LEFT JOIN " +
                "    (SELECT SpecificationItem.SpecificationItemID, SpecificationItem.Name, SpecificationItemLocale.DisplayName, SpecificationGroup.SpecificationGroupName FROM SpecificationItem " +
                "     LEFT JOIN (SELECT SpecificationItemLocale.SpecificationItemID, SpecificationItemLocale.DisplayName " +
                "         FROM  SpecificationItemLocale WHERE CultureID = @CultureID) AS SpecificationItemLocale " +
                "     ON SpecificationItem.SpecificationItemID = SpecificationItemLocale.SpecificationItemID " +
                "     LEFT JOIN " +
                "    (SELECT SpecificationGroup.SpecificationGroupID, SpecificationGroup.Name AS SpecificationGroupName " +
                "     FROM SpecificationGroup " +
                "     LEFT JOIN SpecificationGroupLocale " +
                "     ON SpecificationGroup.SpecificationGroupID = SpecificationGroupLocale.SpecificationGroupID) AS SpecificationGroup " +
                "     ON SpecificationItem.SpecificationGroupID = SpecificationGroup.SpecificationGroupID) AS SpecificationItem " +
                "ON GoogleSpecMapping.SpecificationItemID = SpecificationItem.SpecificationItemID " +
                "LEFT JOIN " +
                "(SELECT GoogleFeedTagValueID, GoogleFeedTagValue.GoogleFeedTagID, GoogleFeedTag.TagName " +
                "FROM GoogleFeedTagValue " +
                "LEFT JOIN GoogleFeedTag " +
                "ON GoogleFeedTagValue.GoogleFeedTagID = GoogleFeedTag.GoogleFeedTagID) AS GoogleFeedTag " +
                "ON GoogleSpecMapping.GoogleFeedTagValueID = GoogleFeedTag.GoogleFeedTagValueID ",
                DataAccess.CreateParameterString( culture.CultureID ) );
            return ConvertUtilities.ToInt32( count );
        }

        private string CreateSortByText( string sortBy )
        {
            if (String.IsNullOrEmpty( sortBy ))
                return " ORDER BY GoogleSpecMapping.GoogleFeedTagValueID ";
            else if (sortBy.Trim().ToLower().StartsWith( "GoogleFeedTagValueID" ))
                return " ORDER BY GoogleSpecMapping." + sortBy.Trim() + " ";
            else
                return " ORDER BY " + sortBy + " ";
        }

        private string ConvertFieldName( string fieldName )
        {
            switch (fieldName.ToLower())
            {
                case "GoogleFeedTagValueID":
                    return "GoogleSpecMapping.GoogleFeedTagValueID";
                case "SpecificationItemID":
                    return "GoogleSpecMapping.SpecificationItemID";
                default:
                    return fieldName;
            }
        }

        private string DeleteCondition( string googleFeedTagValueID, string specificationValue )
        {
            if (String.IsNullOrEmpty( googleFeedTagValueID ))
                return " WHERE Value = @Value ";
            else if (String.IsNullOrEmpty( specificationValue ))
                return " WHERE GoogleFeedTagValueID = @GoogleFeedTagValueID ";
            else
                return " WHERE GoogleFeedTagValueID = @GoogleFeedTagValueID AND Value = @Value ";
        }

        #endregion

        #region Public

        public IList<TableSchemaItem> GetTableSchema()
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT GoogleFeedTag.TagName, GoogleFeedTagValue.ValueName, SpecificationItem.DisplayName, Value " +
                "FROM GoogleSpecMapping " +
                "LEFT JOIN " +
                "    (SELECT GoogleFeedTagValueID, ValueName FROM GoogleFeedTagValue) AS GoogleFeedTagValue " +
                "     ON GoogleSpecMapping.GoogleFeedTagValueID = GoogleFeedTagValue.GoogleFeedTagValueID " +
                "     LEFT JOIN " +
                "    (SELECT SpecificationItem.SpecificationItemID, SpecificationItem.Name, SpecificationItemLocale.DisplayName, SpecificationGroup.SpecificationGroupName FROM SpecificationItem " +
                "     LEFT JOIN SpecificationItemLocale " +
                "     ON SpecificationItem.SpecificationItemID = SpecificationItemLocale.SpecificationItemID " +
                "     LEFT JOIN" +
                "    (SELECT SpecificationGroup.SpecificationGroupID, SpecificationGroup.Name AS SpecificationGroupName " +
                "     FROM SpecificationGroup " +
                "     LEFT JOIN SpecificationGroupLocale " +
                "     ON SpecificationGroup.SpecificationGroupID = SpecificationGroupLocale.SpecificationGroupID) AS SpecificationGroup " +
                "     ON SpecificationItem.SpecificationGroupID = SpecificationGroup.SpecificationGroupID) AS SpecificationItem " +
                "ON GoogleSpecMapping.SpecificationItemID = SpecificationItem.SpecificationItemID " +
                "LEFT JOIN " +
                "(SELECT GoogleFeedTagValueID, GoogleFeedTagValue.GoogleFeedTagID, GoogleFeedTag.TagName " +
                "FROM GoogleFeedTagValue " +
                "LEFT JOIN GoogleFeedTag " +
                "ON GoogleFeedTagValue.GoogleFeedTagID = GoogleFeedTag.GoogleFeedTagID) AS GoogleFeedTag " +
                "ON GoogleSpecMapping.GoogleFeedTagValueID = GoogleFeedTag.GoogleFeedTagValueID " +
                "WHERE 1 = 0 " );

            return RepositoryUtilities.ConvertTableToSchemaItemList( table );
        }

        public void Delete( string specificationItemID )
        {
            DataAccess.ExecuteNonQuery(
                "DELETE FROM GoogleSpecMapping " +
                "WHERE SpecificationItemID = @SpecificationItemID ",
                DataAccess.CreateParameterString( specificationItemID ) );
        }

        public void Delete( string googleFeedTagValueID, string specificationValue )
        {
            DataAccess.ExecuteNonQuery(
                "DELETE FROM GoogleSpecMapping " +
                DeleteCondition( googleFeedTagValueID, specificationValue ),
                DataAccess.CreateParameterString( googleFeedTagValueID ),
                DataAccess.CreateParameterString( specificationValue ) );
        }

        public GoogleSpecMapping Save( Culture culture, GoogleSpecMapping googleSpecMapping )
        {
            GoogleSpecMapping result = GetOne( culture, googleSpecMapping.GoogleFeedTagValueID, googleSpecMapping.Value );
            if (result.IsNull)
                return Create( culture, googleSpecMapping );
            else
                return result;
        }

        public GoogleSpecMapping Update( Culture culture, string oldTagValueID, string oldSpecValue, GoogleSpecMapping newGoogleSpecMapping )
        {
            Delete( oldTagValueID, oldSpecValue );
            return Create( culture, newGoogleSpecMapping );
        }

        public GoogleSpecMapping GetOne( Culture culture, string googleFeedTagValueID, string specificationItemValue )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT GoogleSpecMapping.GoogleFeedTagValueID, GoogleSpecMapping.SpecificationItemID, Value, " +
                "GoogleFeedTagValue.ValueName, SpecificationItem.DisplayName, GoogleFeedTag.GoogleFeedTagID, GoogleFeedTag.TagName , SpecificationItem.SpecificationGroupName " +
                "FROM GoogleSpecMapping " +
                "LEFT JOIN " +
                "    (SELECT GoogleFeedTagValueID, ValueName FROM GoogleFeedTagValue) AS GoogleFeedTagValue " +
                "     ON GoogleSpecMapping.GoogleFeedTagValueID = GoogleFeedTagValue.GoogleFeedTagValueID " +
                "     LEFT JOIN " +
                "    (SELECT SpecificationItem.SpecificationItemID, SpecificationItem.Name, SpecificationItemLocale.DisplayName, SpecificationGroup.SpecificationGroupName FROM SpecificationItem " +
                "     LEFT JOIN (SELECT SpecificationItemLocale.SpecificationItemID, SpecificationItemLocale.DisplayName " +
                "         FROM  SpecificationItemLocale WHERE CultureID = @CultureID) AS SpecificationItemLocale " +
                "     ON SpecificationItem.SpecificationItemID = SpecificationItemLocale.SpecificationItemID " +
                "     LEFT JOIN " +
                "    (SELECT SpecificationGroup.SpecificationGroupID, SpecificationGroup.Name AS SpecificationGroupName " +
                "     FROM SpecificationGroup " +
                "     LEFT JOIN SpecificationGroupLocale " +
                "     ON SpecificationGroup.SpecificationGroupID = SpecificationGroupLocale.SpecificationGroupID) AS SpecificationGroup " +
                "     ON SpecificationItem.SpecificationGroupID = SpecificationGroup.SpecificationGroupID) AS SpecificationItem " +
                "ON GoogleSpecMapping.SpecificationItemID = SpecificationItem.SpecificationItemID " +
                "LEFT JOIN " +
                "(SELECT GoogleFeedTagValueID, GoogleFeedTagValue.GoogleFeedTagID, GoogleFeedTag.TagName " +
                "FROM GoogleFeedTagValue " +
                "LEFT JOIN GoogleFeedTag " +
                "ON GoogleFeedTagValue.GoogleFeedTagID = GoogleFeedTag.GoogleFeedTagID) AS GoogleFeedTag " +
                "ON GoogleSpecMapping.GoogleFeedTagValueID = GoogleFeedTag.GoogleFeedTagValueID " +
                "WHERE GoogleSpecMapping.GoogleFeedTagValueID = @GoogleFeedTagValueID AND Value = @Value  ",
                DataAccess.CreateParameterString( culture.CultureID ),
                DataAccess.CreateParameterString( googleFeedTagValueID ),
                DataAccess.CreateParameterString( specificationItemValue ) );

            if (table.Rows.Count > 0)
                return CreateOneFromRow( table.Rows[0] );
            else
                return GoogleSpecMapping.Null;
        }

        public IList<GoogleSpecMapping> GetAll( Culture culture )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT GoogleSpecMapping.GoogleFeedTagValueID, GoogleSpecMapping.SpecificationItemID, Value, " +
                "GoogleFeedTagValue.ValueName, SpecificationItem.DisplayName, GoogleFeedTag.GoogleFeedTagID, GoogleFeedTag.TagName , SpecificationItem.SpecificationGroupName " +
                "FROM GoogleSpecMapping " +
                "LEFT JOIN " +
                "    (SELECT GoogleFeedTagValueID, ValueName FROM GoogleFeedTagValue) AS GoogleFeedTagValue " +
                "     ON GoogleSpecMapping.GoogleFeedTagValueID = GoogleFeedTagValue.GoogleFeedTagValueID " +
                "     LEFT JOIN " +
                "    (SELECT SpecificationItem.SpecificationItemID, SpecificationItem.Name, SpecificationItemLocale.DisplayName, SpecificationGroup.SpecificationGroupName FROM SpecificationItem " +
                "     LEFT JOIN (SELECT SpecificationItemLocale.SpecificationItemID, SpecificationItemLocale.DisplayName " +
                "         FROM  SpecificationItemLocale WHERE CultureID = @CultureID) AS SpecificationItemLocale " +
                "     ON SpecificationItem.SpecificationItemID = SpecificationItemLocale.SpecificationItemID " +
                "     LEFT JOIN " +
                "    (SELECT SpecificationGroup.SpecificationGroupID, SpecificationGroup.Name AS SpecificationGroupName " +
                "     FROM SpecificationGroup " +
                "     LEFT JOIN SpecificationGroupLocale " +
                "     ON SpecificationGroup.SpecificationGroupID = SpecificationGroupLocale.SpecificationGroupID) AS SpecificationGroup " +
                "     ON SpecificationItem.SpecificationGroupID = SpecificationGroup.SpecificationGroupID) AS SpecificationItem " +
                "ON GoogleSpecMapping.SpecificationItemID = SpecificationItem.SpecificationItemID " +
                "LEFT JOIN " +
                "(SELECT GoogleFeedTagValueID, GoogleFeedTagValue.GoogleFeedTagID, GoogleFeedTag.TagName " +
                "FROM GoogleFeedTagValue " +
                "LEFT JOIN GoogleFeedTag " +
                "ON GoogleFeedTagValue.GoogleFeedTagID = GoogleFeedTag.GoogleFeedTagID) AS GoogleFeedTag " +
                "ON GoogleSpecMapping.GoogleFeedTagValueID = GoogleFeedTag.GoogleFeedTagValueID ",
                DataAccess.CreateParameterString( culture.CultureID ) );

            return ConvertTableToList( table );
        }

        public IList<GoogleSpecMapping> SearchGoogleSpecMapping(
            Culture culture,
            string sortBy,
            SearchFilter searchFilter,
            int startIndex,
            int endIndex,
            out int howManyItems )
        {
            ArrayList parameterList = new ArrayList();
            parameterList.Add( DataAccess.CreateParameterString( RepositoryUtilities.GetCultureID( culture ) ) );
            SearchFilterHelperNew searchFilterHelper = new SearchFilterHelperNew();
            string optionFieldName = ConvertFieldName( searchFilter.FieldName );
            string commandSearch = searchFilterHelper.CreateCommandWithWhere( searchFilter, optionFieldName, parameterList );

            string query = "SELECT ROW_NUMBER() over ( " + CreateSortByText( sortBy ) + ") as rownum, " +
                " * FROM (" +
                "SELECT GoogleSpecMapping.GoogleFeedTagValueID, GoogleSpecMapping.SpecificationItemID, Value, " +
                "GoogleFeedTagValue.ValueName, SpecificationItem.DisplayName, GoogleFeedTag.GoogleFeedTagID, GoogleFeedTag.TagName, " +
                "SpecificationItem.SpecificationGroupName, (SpecificationItem.SpecificationGroupName + ' - ' + SpecificationItem.DisplayName ) as SpecificationGroupDisplayName " +
                "FROM GoogleSpecMapping " +
                "LEFT JOIN " +
                "    (SELECT GoogleFeedTagValueID, ValueName FROM GoogleFeedTagValue) AS GoogleFeedTagValue " +
                "     ON GoogleSpecMapping.GoogleFeedTagValueID = GoogleFeedTagValue.GoogleFeedTagValueID " +
                "     LEFT JOIN " +
                "    (SELECT SpecificationItem.SpecificationItemID, SpecificationItem.Name, SpecificationItemLocale.DisplayName, SpecificationGroup.SpecificationGroupName FROM SpecificationItem " +
                "     LEFT JOIN (SELECT SpecificationItemLocale.SpecificationItemID, SpecificationItemLocale.DisplayName " +
                "         FROM  SpecificationItemLocale WHERE CultureID = @CultureID) AS SpecificationItemLocale " +
                "     ON SpecificationItem.SpecificationItemID = SpecificationItemLocale.SpecificationItemID " +
                "     LEFT JOIN " +
                "    (SELECT SpecificationGroup.SpecificationGroupID, SpecificationGroup.Name AS SpecificationGroupName " +
                "     FROM SpecificationGroup " +
                "     LEFT JOIN SpecificationGroupLocale " +
                "     ON SpecificationGroup.SpecificationGroupID = SpecificationGroupLocale.SpecificationGroupID) AS SpecificationGroup " +
                "     ON SpecificationItem.SpecificationGroupID = SpecificationGroup.SpecificationGroupID) AS SpecificationItem " +
                "ON GoogleSpecMapping.SpecificationItemID = SpecificationItem.SpecificationItemID " +
                "LEFT JOIN " +
                "(SELECT GoogleFeedTagValueID, GoogleFeedTagValue.GoogleFeedTagID, GoogleFeedTag.TagName " +
                "FROM GoogleFeedTagValue " +
                "LEFT JOIN GoogleFeedTag " +
                "ON GoogleFeedTagValue.GoogleFeedTagID = GoogleFeedTag.GoogleFeedTagID) AS GoogleFeedTag " +
                "ON GoogleSpecMapping.GoogleFeedTagValueID = GoogleFeedTag.GoogleFeedTagValueID " +
                commandSearch + " ) AS GoogleSpecMapping ";

            string extQuery = "with SearchResult as (" + query + ") " +
                    " select * from SearchResult where rownum between " + (startIndex + 1).ToString() + " and " + (endIndex + 1).ToString() + ";";

            DataTable full = DataAccess.ExecuteSelect( extQuery,
                DataAccess.ConvertArrayListToDbParameters( parameterList ) );
            howManyItems = 0;
            howManyItems = GetGoogleSpecMappingCount( culture );

            return ConvertTableToList( full, startIndex, endIndex );
        }

        #endregion
    }
}
