using System;
using System.Data;
using System.Collections.Generic;
using System.Configuration;
using System.IO;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using Vevo;
using Vevo.Domain;
using Vevo.Domain.Orders;
using Vevo.Shared.Utilities;
using Vevo.WebAppLib;
using System.Collections;
using Vevo.Data;

namespace Vevo.WebUI
{
    /// <summary>
    /// Summary description for EmailTemplates
    /// </summary>
    public static class EmailTemplates
    {
        // Constances for email templates
        public const string TemplateFolder = "ContentTemplates/";
        public const string NewsletterBodyFileName = "NewsletterBody_Html.txt";


        #region Private

        private static string GetPath( string fileName )
        {
            return HttpContext.Current.Server.MapPath( "~/" ) + TemplateFolder + "/" + fileName;
        }

        private static string ReadFile( string fileName )
        {
            string bodyMail = String.Empty;
            using (StreamReader readText = File.OpenText( GetPath( fileName ) ))
            {
                bodyMail = readText.ReadToEnd();
            }
            return bodyMail;
        }

        #endregion


        #region Constructors

        public static bool ReadTemplate(
            string subjectFileName,
            string bodyFileName,
            out string subject,
            out string body )
        {
            subject = String.Empty;
            body = String.Empty;
            try
            {
                subject = ReadFile( subjectFileName );
                body = ReadFile( bodyFileName );
                if (body.ToLower().IndexOf( "<br" ) == -1 && 
                    body.ToLower().IndexOf( "<p" ) == -1 )
                {
                    body = body.Replace( "\n", "<BR />" );
                }

                return true;
            }
            catch
            {
                return false;
            }
        }

        #endregion


        #region Public Methods

        public static bool ReadTemplate(
            string fileName,
            out string data )
        {
            data = String.Empty;
            try
            {
                data = ReadFile( fileName );
                return true;
            }
            catch
            {
                return false;
            }
        }

        public static string ReadAdminTemplate( string filename )
        {
            return ReadFile( filename );
        }

        public static bool ReadDownloadTemplate(
            out string subject,
            out string body )
        {
            return ReadTemplate(
                "DownloadFileLinksSubject.txt", "DownloadFileLinksBody.txt",
                out subject, out body );
        }

        public static string ReadInvoiceTemplate( string orderID, Culture culture, Currency currency )
        {
            OrderRepository orderRepository = new OrderRepository();
            Order order = orderRepository.GetOne( orderID );

            string bodyMail;

            EmailTemplateTextVariable.ReplaceInvoiceText( order, culture, currency, out bodyMail );

            return bodyMail;
        }

        public static DataTable GetTemplateList()
        {
            DataSet ds = new DataSet();
            string pathFile = HttpContext.Current.Server.MapPath( "~/ContentTemplates/EmailTemplateList.xml" );
            if (File.Exists( pathFile ))
            {
                ds.ReadXml( pathFile );
            }
            else
                throw new Exception( "File Not Found : " + pathFile );

            return ds.Tables[0];
        }

        #endregion
    }
}
