using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using Vevo;
using Vevo.Domain;
using Vevo.Domain.Orders;
using Vevo.Domain.Payments;
using Vevo.WebUI;
using Vevo.Domain.Stores;
using Vevo.Deluxe.Domain.GiftRegistry;
using Vevo.Deluxe.Domain.CustomerReward;
using Vevo.Domain.Users;
using Vevo.Deluxe.Domain.Orders;

namespace Vevo.WebUI.Orders
{
    /// <summary>
    /// Summary description for BaseProcessCheckoutPage
    /// </summary>
    public class BaseProcessCheckoutPage : Vevo.WebUI.International.BaseLanguagePage
    {
        public BaseProcessCheckoutPage()
        {
            Load += new EventHandler( BaseProcessCheckoutPage_Load );
        }

        public string GetPage()
        {
            string[] splitURL = Request.Url.AbsoluteUri.Split( '/' );
            int upperBound = splitURL.GetUpperBound( 0 );

            return splitURL[upperBound];
        }

        private void BaseProcessCheckoutPage_Load( object sender, EventArgs e )
        {
            if (!Page.User.Identity.IsAuthenticated && !(Request.QueryString["skiplogin"] == "true"))
            {
                CheckoutDetails details = StoreContext.CheckoutDetails;
                PaymentOption paymentOption = DataAccessContext.PaymentOptionRepository.GetOne( StoreContext.Culture, details.PaymentMethod.Name );
                if (!paymentOption.AnonymousCheckoutAllowed)
                    Response.Redirect( "~/UserLogin.aspx?ReturnUrl=Checkout.aspx" );
            }
        }

        protected Order PlaceOrder( OrderCreateService orderCreateService,out OrderAmount amount)
        {
            amount = orderCreateService.GetOrderAmount( StoreContext.Customer )
                .Add( CartItemPromotion.CalculatePromotionShippingAndTax(
                StoreContext.CheckoutDetails,
                StoreContext.ShoppingCart.SeparateCartItemGroups(),
                StoreContext.Customer ) );
            Order order = orderCreateService.PlaceOrder( amount,StoreContext.Customer, new StoreRetriever(), StoreContext.Culture );
            GiftRegistry.UpdateGiftRegistryQuantity( StoreContext.ShoppingCart, StoreContext.CheckoutDetails );
            CustomerRewardPoint.UpdateRedeemPoint( StoreContext.CheckoutDetails, StoreContext.Customer, order );
            return order;
        }

        protected Order PlaceOrderAnonymous( string userName, OrderCreateService orderCreateService, CheckoutDetails checkout,out OrderAmount amount )
        {
            amount = orderCreateService.GetOrderAmount( Customer.Null )
                .Add( CartItemPromotion.CalculatePromotionShippingAndTax(
                StoreContext.CheckoutDetails,
                StoreContext.ShoppingCart.SeparateCartItemGroups(),
                Customer.Null ) );

            Order order = orderCreateService.PlaceOrderAnonymous( amount,
                SystemConst.AnonymousUser,
                checkout.BillingAddress,
                checkout.Email, new StoreRetriever(), StoreContext.Culture );

            return order;
        }
    }
}